from django.test import TestCase

from ..middleware import RealIPMiddleware


class MockRequest:
    def __init__(self, addr, forwarded_for=None):
        self.META = {"REMOTE_ADDR": addr}

        if forwarded_for:
            self.META["HTTP_X_FORWARDED_FOR"] = forwarded_for


class RealIPMiddlewareTests(TestCase):
    def test_middleware_sets_ip_from_remote_add(self):
        """Middleware sets ip from remote_addr header"""
        request = MockRequest("83.42.13.77")
        RealIPMiddleware().process_request(request)

        self.assertEqual(request.user_ip, request.META["REMOTE_ADDR"])

    def test_middleware_sets_ip_from_forwarded_for(self):
        """Middleware sets ip from forwarded_for header"""
        request = MockRequest("127.0.0.1", "83.42.13.77")
        RealIPMiddleware().process_request(request)

        self.assertEqual(request.user_ip, request.META["HTTP_X_FORWARDED_FOR"])



from django.conf.urls import include, url

from ...core.views import home_redirect

from ..views import (
    activation,
    auth,
    avatarserver,
    forgottenpassword,
    lists,
    options,
    profile,
)

urlpatterns = [
    url(r"^banned/$", home_redirect, name="banned"),
    url(r"^login/$", auth.login, name="login"),
    url(r"^logout/$", auth.logout, name="logout"),
    url(
        r"^request-activation/$",
        activation.request_activation,
        name="request-activation",
    ),
    url(
        r"^activation/(?P<pk>\d+)/(?P<token>[a-zA-Z0-9]+)/$",
        activation.activate_by_token,
        name="activate-by-token",
    ),
    url(
        r"^forgotten-password/$",
        forgottenpassword.request_reset,
        name="forgotten-password",
    ),
    url(
        r"^forgotten-password/(?P<pk>\d+)/(?P<token>[a-zA-Z0-9]+)/$",
        forgottenpassword.reset_password_form,
        name="forgotten-password-change-form",
    ),
]

urlpatterns += [
    url(r"^options/$", options.index, name="options"),
    url(r"^options/(?P<form_name>[-a-zA-Z]+)/$", options.index, name="options-form"),
    url(r"^options/forum-options/$", options.index, name="usercp-change-forum-options"),
    url(r"^options/edit-details/$", options.index, name="usercp-edit-details"),
    url(r"^options/change-username/$", options.index, name="usercp-change-username"),
    url(
        r"^options/sign-in-credentials/$",
        options.index,
        name="usercp-change-email-password",
    ),
    url(
        r"^options/change-email/(?P<token>[a-zA-Z0-9]+)/$",
        options.confirm_email_change,
        name="options-confirm-email-change",
    ),
    url(
        r"^options/change-password/(?P<token>[a-zA-Z0-9]+)/$",
        options.confirm_password_change,
        name="options-confirm-password-change",
    ),
    url(r"^options/dowload-data/$", options.index, name="usercp-download-data"),
    url(r"^options/delete-account/$", options.index, name="usercp-delete-account"),
]

urlpatterns += [
    url(
        r"^users/",
        include(
            [
                url(r"^$", lists.landing, name="users"),
                url(
                    r"^active-posters/$",
                    lists.ActivePostersView.as_view(),
                    name="users-active-posters",
                ),
                url(
                    r"^(?P<slug>[-a-zA-Z0-9]+)/$",
                    lists.RankUsersView.as_view(),
                    name="users-rank",
                ),
                url(
                    r"^(?P<slug>[-a-zA-Z0-9]+)/(?P<page>\d+)/$",
                    lists.RankUsersView.as_view(),
                    name="users-rank",
                ),
            ]
        ),
    )
]

urlpatterns += [
    url(
        r"^u/(?P<slug>[a-zA-Z0-9]+)/(?P<pk>\d+)/",
        include(
            [
                url(r"^$", profile.LandingView.as_view(), name="user"),
                url(r"^posts/$", profile.UserPostsView.as_view(), name="user-posts"),
                url(
                    r"^threads/$",
                    profile.UserThreadsView.as_view(),
                    name="user-threads",
                ),
                url(
                    r"^followers/$",
                    profile.UserFollowersView.as_view(),
                    name="user-followers",
                ),
                url(
                    r"^follows/$",
                    profile.UserFollowsView.as_view(),
                    name="user-follows",
                ),
                url(
                    r"^details/$",
                    profile.UserProfileDetailsView.as_view(),
                    name="user-details",
                ),
                url(
                    r"^username-history/$",
                    profile.UserUsernameHistoryView.as_view(),
                    name="username-history",
                ),
                url(r"^ban-details/$", profile.UserBanView.as_view(), name="user-ban"),
            ]
        ),
    )
]

urlpatterns += [
    url(r"^avatar/$", avatarserver.blank_avatar, name="blank-avatar"),
    url(
        r"^avatar/(?P<pk>\d+)/(?P<size>\d+)/$",
        avatarserver.user_avatar,
        name="user-avatar",
    ),
]



XXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXX
XXXXXXXXXXXX
XXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX



from ariadne.contrib.django.views import GraphQLView
from django.conf.urls import url

from .schema import schema


class MisagoAdminExtension:
    def register_urlpatterns(self, urlpatterns):
        # GraphQL API
        urlpatterns.namespace(r"^graphql/", "graphql")
        urlpatterns.patterns(
            "graphql", url(r"^$", GraphQLView.as_view(schema=schema), name="index")
        )



from .audittrail import create_user_audit_trail
from .avatars import set_default_avatar
from .models import User


def setup_new_user(settings, user):
    set_default_subscription_options(settings, user)

    set_default_avatar(
        user, settings.default_avatar, settings.default_gravatar_fallback
    )

    if user.joined_from_ip:
        create_user_audit_trail(user, user.joined_from_ip, user)


SUBSCRIPTION_CHOICES = {
    "no": User.SUBSCRIPTION_NONE,
    "watch": User.SUBSCRIPTION_NOTIFY,
    "watch_email": User.SUBSCRIPTION_ALL,
}


def set_default_subscription_options(settings, user):
    started_threads = SUBSCRIPTION_CHOICES[settings.subscribe_start]
    user.subscribe_to_started_threads = started_threads

    replied_threads = SUBSCRIPTION_CHOICES[settings.subscribe_reply]
    user.subscribe_to_replied_threads = replied_threads



from django.conf.urls import url
from django.utils.translation import gettext_lazy as _

from .views import (
    AgreementsList,
    DeleteAgreement,
    DisableAgreement,
    EditAgreement,
    NewAgreement,
    SetAgreementAsActive,
)


class MisagoAdminExtension:
    def register_urlpatterns(self, urlpatterns):
        # Legal Agreements
        urlpatterns.namespace(r"^agreements/", "agreements", "settings")
        urlpatterns.patterns(
            "settings:agreements",
            url(r"^$", AgreementsList.as_view(), name="index"),
            url(r"^(?P<page>\d+)/$", AgreementsList.as_view(), name="index"),
            url(r"^new/$", NewAgreement.as_view(), name="new"),
            url(r"^edit/(?P<pk>\d+)/$", EditAgreement.as_view(), name="edit"),
            url(r"^delete/(?P<pk>\d+)/$", DeleteAgreement.as_view(), name="delete"),
            url(
                r"^set-as-active/(?P<pk>\d+)/$",
                SetAgreementAsActive.as_view(),
                name="set-as-active",
            ),
            url(r"^disable/(?P<pk>\d+)/$", DisableAgreement.as_view(), name="disable"),
        )

    def register_navigation_nodes(self, site):
        site.add_node(
            name=_("Legal agreements"),
            description=_("Set terms of service and privacy policy contents."),
            parent="settings",
            namespace="agreements",
        )



from django.conf.urls import url
from django.utils.translation import gettext_lazy as _

from .views import icons_admin


class MisagoAdminExtension:
    def register_urlpatterns(self, urlpatterns):
        # Icons
        urlpatterns.namespace(r"^icons/", "icons", "settings")
        urlpatterns.patterns("settings:icons", url(r"^$", icons_admin, name="index"))

    def register_navigation_nodes(self, site):
        site.add_node(
            name=_("Icons"),
            description=_("Upload favicon and application icon for the site."),
            parent="settings",
            namespace="icons",
        )



from django.conf.urls import url

from ..views import privacy_policy, terms_of_service

urlpatterns = [
    url(r"^privacy-policy/$", privacy_policy, name="privacy-policy"),
    url(r"^terms-of-service/$", terms_of_service, name="terms-of-service"),
]



from django.conf.urls import url

from . import views


urlpatterns = [
    url(r"^login/(?P<backend>[^/]+)/$", views.auth, name="social-begin"),
    url(r"^complete/(?P<backend>[^/]+)/$", views.complete, name="social-complete"),
]



import markdown
from markdown.inlinepatterns import LinkPattern
from markdown.util import etree

IMAGES_RE = r"\!(\s?)\((<.*?>|([^\)]*))\)"


class ShortImagesExtension(markdown.Extension):
    def extendMarkdown(self, md):
        md.registerExtension(self)
        md.inlinePatterns.add(
            "misago_short_images", ShortImagePattern(IMAGES_RE, md), "_end"
        )


class ShortImagePattern(LinkPattern):
    def handleMatch(self, m):
        img_src = m.groups()[2].strip()
        if img_src:
            img = etree.Element("img")
            img.set("src", img_src)
            img.set("alt", img_src)
            return img



import bleach
import markdown
from bs4 import BeautifulSoup
from django.http import Http404
from django.urls import resolve
from htmlmin.minify import html_minify
from markdown.extensions.fenced_code import FencedCodeExtension

from ..conf import settings
from .bbcode import blocks, inline
from .md.shortimgs import ShortImagesExtension
from .md.striketrough import StriketroughExtension
from .mentions import add_mentions
from .pipeline import pipeline

MISAGO_ATTACHMENT_VIEWS = ("misago:attachment", "misago:attachment-thumbnail")


def parse(
    text,
    request,
    poster,
    allow_mentions=True,
    allow_links=True,
    allow_images=True,
    allow_blocks=True,
    force_shva=False,
    minify=True,
):
    """
    Message parser

    Utility for flavours to call

    Breaks text into paragraphs, supports code, spoiler and quote blocks,
    headers, lists, images, spoilers, text styles

    Returns dict object
    """
    md = md_factory(
        allow_links=allow_links, allow_images=allow_images, allow_blocks=allow_blocks
    )

    parsing_result = {
        "original_text": text,
        "parsed_text": "",
        "markdown": md,
        "mentions": [],
        "images": [],
        "internal_links": [],
        "outgoing_links": [],
    }

    # Parse text
    parsed_text = md.convert(text)

    # Clean and store parsed text
    parsing_result["parsed_text"] = parsed_text.strip()

    if allow_links:
        linkify_paragraphs(parsing_result)

    parsing_result = pipeline.process_result(parsing_result)

    if allow_mentions:
        add_mentions(request, parsing_result)

    if allow_links or allow_images:
        clean_links(request, parsing_result, force_shva)

    if minify:
        minify_result(parsing_result)
    return parsing_result


def md_factory(allow_links=True, allow_images=True, allow_blocks=True):
    """creates and configures markdown object"""
    md = markdown.Markdown(extensions=["markdown.extensions.nl2br"])

    # Remove HTML allowances
    del md.preprocessors["html_block"]
    del md.inlinePatterns["html"]

    # Remove references
    del md.preprocessors["reference"]
    del md.inlinePatterns["reference"]
    del md.inlinePatterns["image_reference"]
    del md.inlinePatterns["short_reference"]

    # Add [b], [i], [u]
    md.inlinePatterns.add("bb_b", inline.bold, "<strong")
    md.inlinePatterns.add("bb_i", inline.italics, "<emphasis")
    md.inlinePatterns.add("bb_u", inline.underline, "<emphasis2")

    # Add ~~deleted~~
    striketrough_md = StriketroughExtension()
    striketrough_md.extendMarkdown(md)

    if allow_links:
        # Add [url]
        md.inlinePatterns.add("bb_url", inline.url(md), "<link")
    else:
        # Remove links
        del md.inlinePatterns["link"]
        del md.inlinePatterns["autolink"]
        del md.inlinePatterns["automail"]

    if allow_images:
        # Add [img]
        md.inlinePatterns.add("bb_img", inline.image(md), "<image_link")
        short_images_md = ShortImagesExtension()
        short_images_md.extendMarkdown(md)
    else:
        # Remove images
        del md.inlinePatterns["image_link"]

    if allow_blocks:
        # Add [hr] and [quote] blocks
        md.parser.blockprocessors.add(
            "bb_hr", blocks.BBCodeHRProcessor(md.parser), ">hr"
        )

        fenced_code = FencedCodeExtension()
        fenced_code.extendMarkdown(md, None)

        code_bbcode = blocks.CodeBlockExtension()
        code_bbcode.extendMarkdown(md)

        quote_bbcode = blocks.QuoteExtension()
        quote_bbcode.extendMarkdown(md)
    else:
        # Remove blocks
        del md.parser.blockprocessors["hashheader"]
        del md.parser.blockprocessors["setextheader"]
        del md.parser.blockprocessors["code"]
        del md.parser.blockprocessors["quote"]
        del md.parser.blockprocessors["hr"]
        del md.parser.blockprocessors["olist"]
        del md.parser.blockprocessors["ulist"]

    return pipeline.extend_markdown(md)


def linkify_paragraphs(result):
    result["parsed_text"] = bleach.linkify(
        result["parsed_text"],
        callbacks=settings.MISAGO_BLEACH_CALLBACKS,
        skip_tags=["a", "code", "pre"],
        parse_email=True,
    )


def clean_links(request, result, force_shva=False):
    host = request.get_host()

    soup = BeautifulSoup(result["parsed_text"], "html5lib")
    for link in soup.find_all("a"):
        if is_internal_link(link["href"], host):
            link["href"] = clean_internal_link(link["href"], host)
            result["internal_links"].append(link["href"])
            link["href"] = clean_attachment_link(link["href"], force_shva)
        else:
            result["outgoing_links"].append(clean_link_prefix(link["href"]))
            link["href"] = assert_link_prefix(link["href"])
            link["rel"] = "nofollow noopener"

        if link.string:
            link.string = clean_link_prefix(link.string)

    for img in soup.find_all("img"):
        img["alt"] = clean_link_prefix(img["alt"])
        if is_internal_link(img["src"], host):
            img["src"] = clean_internal_link(img["src"], host)
            result["images"].append(img["src"])
            img["src"] = clean_attachment_link(img["src"], force_shva)
        else:
            result["images"].append(clean_link_prefix(img["src"]))
            img["src"] = assert_link_prefix(img["src"])

    # [6:-7] trims <body></body> wrap
    result["parsed_text"] = str(soup.body)[6:-7]


def is_internal_link(link, host):
    if link.startswith("/") and not link.startswith("//"):
        return True

    link = clean_link_prefix(link).lstrip("www.").lower()
    return link.lower().startswith(host.lstrip("www."))


def clean_link_prefix(link):
    if link.lower().startswith("https:"):
        link = link[6:]
    if link.lower().startswith("http:"):
        link = link[5:]
    if link.startswith("//"):
        link = link[2:]
    return link


def assert_link_prefix(link):
    if link.lower().startswith("https:"):
        return link
    if link.lower().startswith("http:"):
        return link
    if link.startswith("//"):
        return "http:%s" % link

    return "http://%s" % link


def clean_internal_link(link, host):
    link = clean_link_prefix(link)

    if link.lower().startswith("www."):
        link = link[4:]
    if host.lower().startswith("www."):
        host = host[4:]

    if link.lower().startswith(host):
        link = link[len(host) :]

    return link or "/"


def clean_attachment_link(link, force_shva=False):
    try:
        resolution = resolve(link)
        url_name = ":".join(resolution.namespaces + [resolution.url_name])
    except (Http404, ValueError):
        return link

    if url_name in MISAGO_ATTACHMENT_VIEWS:
        if force_shva:
            link = "%s?shva=1" % link
        elif link.endswith("?shva=1"):
            link = link[:-7]
    return link


def minify_result(result):
    result["parsed_text"] = html_minify(result["parsed_text"])
    result["parsed_text"] = strip_html_head_body(result["parsed_text"])


def strip_html_head_body(parsed_text):
    # [25:-14] trims <html><head></head><body> and </body></html>
    return parsed_text[25:-14]



from django.urls import reverse

from ...users.test import AuthenticatedUserTestCase


class ParseMarkupApiTests(AuthenticatedUserTestCase):
    def setUp(self):
        super().setUp()

        self.api_link = reverse("misago:api:parse-markup")

    def test_is_anonymous(self):
        """api requires authentication"""
        self.logout_user()

        response = self.client.post(self.api_link)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(
            response.json(), {"detail": "This action is not available to guests."}
        )

    def test_no_data(self):
        """api handles no data"""
        response = self.client.post(self.api_link)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {"detail": "You have to enter a message."})

    def test_invalid_data(self):
        """api handles post that is invalid type"""
        response = self.client.post(
            self.api_link, "[]", content_type="application/json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {"detail": "Invalid data. Expected a dictionary, but got list."},
        )

        response = self.client.post(
            self.api_link, "123", content_type="application/json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {"detail": "Invalid data. Expected a dictionary, but got int."},
        )

        response = self.client.post(
            self.api_link, '"string"', content_type="application/json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {"detail": "Invalid data. Expected a dictionary, but got str."},
        )

        response = self.client.post(
            self.api_link, "malformed", content_type="application/json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {"detail": "JSON parse error - Expecting value: line 1 column 1 (char 0)"},
        )

    def test_empty_post(self):
        """api handles empty post"""
        response = self.client.post(self.api_link, {"post": ""})
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {"detail": "You have to enter a message."})

        # regression test for #929
        response = self.client.post(self.api_link, {"post": "\n"})
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {"detail": "You have to enter a message."})

    def test_invalid_post(self):
        """api handles invalid post type"""
        response = self.client.post(self.api_link, {"post": 123})
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {
                "detail": (
                    "Posted message should be at least 5 characters long (it has 3)."
                )
            },
        )

    def test_valid_post(self):
        """api returns parsed markup for valid post"""
        response = self.client.post(self.api_link, {"post": "Lorem ipsum dolor met!"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {"parsed": "<p>Lorem ipsum dolor met!</p>"})



from django.urls import reverse


def test_view_begins_social_auth_for_provider(client, provider):
    response = client.get(
        reverse("misago:social-begin", kwargs={"backend": provider.pk})
    )
    assert response.status_code == 302


def test_view_returns_404_for_disabled_provider(client, disabled_provider):
    response = client.get(
        reverse("misago:social-begin", kwargs={"backend": disabled_provider.pk})
    )
    assert response.status_code == 404


def test_view_returns_404_for_undefined_provider(db, client):
    response = client.get(
        reverse("misago:social-begin", kwargs={"backend": "undefined"})
    )
    assert response.status_code == 404



from django.conf.urls import url

from ...conf import settings
from ...core.views import home_redirect

from ..views import categories

if settings.MISAGO_THREADS_ON_INDEX:
    URL_PATH = r"^categories/$"
else:
    URL_PATH = r"^$"

urlpatterns = [
    url(URL_PATH, categories, name="categories"),
    # fallback for after we changed index setting
    url(r"^categories/$", home_redirect),
]
